/**
 * @file eth_lwip.c
 * @date 2017-06-26
 *
 * NOTE:
 * This file is generated by DAVE. Any manual modification done to this file will be lost when the code is regenerated.
 *
 * @cond
 ***********************************************************************************************************************
 * ETH_LWIP v4.1.16 - Initializes LWIP stack.
 *
 * Copyright (c) 2015-2019, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * To improve the quality of the software, users are encouraged to share modifications, enhancements or bug fixes
 * with Infineon Technologies AG (dave@infineon.com).
 ***********************************************************************************************************************
 *
 * Change History
 * --------------
 *
 * 2016-01-14:
 *     - Initial version for DAVEv4
 *
 * 2017-06-26:
 *     - Changes required by lwIP 2.0.2
 *
 * @endcond
 *
 */

/***********************************************************************************************************************
 * HEADER FILES
 **********************************************************************************************************************/
#include "eth_lwip.h"

#include "ETH_LWIP/lwip/include/lwip/netif.h"
#include "ETH_LWIP/lwip/port/netif/ethernetif.h"
#include "ETH_LWIP/lwip/include/netif/etharp.h"

/*Extern declaration for objects defined in ethernetif.c*/
extern XMC_ETH_MAC_t eth_mac;
extern struct netif xnetif;
/***********************************************************************************************************************
 * MACROS
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * LOCAL DATA
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * LOCAL ROUTINES
 **********************************************************************************************************************/
#if NO_SYS == 0
static void lwip_linit(void *arg);
#endif
/**********************************************************************************************************************
 * API IMPLEMENTATION
 **********************************************************************************************************************/
/*
 * @brief API to retrieve the version of the ETH_LWIP APP.
 *
 * @return DAVE_APP_VERSION_t Structure containing major version, minor version
 *         and patch version.
 */
DAVE_APP_VERSION_t ETH_LWIP_GetAppVersion(void)
{
  DAVE_APP_VERSION_t version;

  version.major = ETH_LWIP_MAJOR_VERSION;
  version.minor = ETH_LWIP_MINOR_VERSION;
  version.patch = ETH_LWIP_PATCH_VERSION;

  return version;
}


/*
 * @brief API to initialize the LWIP software stack.
 *
 * @param[in]  handle ETH_LWIP APP handle pointer of type ETH_LWIP_t*
 *
 * @return  ETH_LWIP_STATUS_t
 *          ETH_LWIP_SUCCESS: for successful LWIP initialization.<BR>
 *          ETH_LWIP_STATUS_FAILURE  : If LWIP initialization fails.<BR>
 *
 */
/*If RTOS is used*/
#if NO_SYS == 0
ETH_LWIP_STATUS_t ETH_LWIP_Init(ETH_LWIP_t* handle)
{
  ETH_LWIP_STATUS_t status = ETH_LWIP_STATUS_SUCCESS;

  if (handle->initialized == false)
  {
    /*Initialize RTOS APP*/
    status = CMSIS_RTOS_Init(&ETH_RTOS_NAME);

    lwip_linit(NULL);

    status = ETH_LWIP_STATUS_SUCCESS;
    handle->initialized = true;
    handle->eth_mac = &eth_mac;
    handle->xnetif = &xnetif;
  }
  return status;
}
#else
ETH_LWIP_STATUS_t ETH_LWIP_Init(ETH_LWIP_t* handle)
{
  ETH_LWIP_STATUS_t status = ETH_LWIP_STATUS_SUCCESS;

  ip_addr_t ipaddr;
  ip_addr_t netmask;
  ip_addr_t gw;

  if (handle->initialized == false)
  {
    /* IP address setting */
#if ((LWIP_DHCP == 1) || (LWIP_AUTOIP == 1))
    ipaddr.addr = 0;
    netmask.addr = 0;
    gw.addr = 0;
#else
    IP4_ADDR(&ipaddr, IP_ADDR0, IP_ADDR1, IP_ADDR2, IP_ADDR3);
    IP4_ADDR(&netmask, NETMASK_ADDR0, NETMASK_ADDR1 , NETMASK_ADDR2, NETMASK_ADDR3);
    IP4_ADDR(&gw, GW_ADDR0, GW_ADDR1, GW_ADDR2, GW_ADDR3);
#endif
    /*Initialize SYSTIMER APP*/
    status = (ETH_LWIP_STATUS_t)SYSTIMER_Init(&ETH_SYSTIMER_NAME);

    lwip_init();

    /* Adds network interface to the netif_list */
    (void)netif_add(&xnetif, &ipaddr, &netmask, &gw, NULL, &ethernetif_init, &ethernet_input);

    /*  Registers the default network interface.*/
    netif_set_default(&xnetif);

    /* If callback enabled */
#if LWIP_NETIF_STATUS_CALLBACK == 1
    /* Initialize interface status change callback */
    netif_set_status_callback(&xnetif, ETH_NETIF_STATUS_CB_FUNCTION);
#endif

    handle->initialized = true;
    handle->eth_mac = &eth_mac;
    handle->xnetif = &xnetif;
  }
  return status;
}
#endif /*NO_SYS*/


/*If RTOS is used*/
#if NO_SYS == 0
/*Local function to initialize LWIP*/
static void lwip_linit(void *arg)
{
  XMC_UNUSED_ARG(arg);

  ip_addr_t ipaddr;
  ip_addr_t netmask;
  ip_addr_t gw;

  /* IP address setting */
#if ((LWIP_DHCP == 1) || (LWIP_AUTOIP == 1))
  ipaddr.addr = 0;
  netmask.addr = 0;
  gw.addr = 0;
#else
  IP4_ADDR(&ipaddr, IP_ADDR0, IP_ADDR1, IP_ADDR2, IP_ADDR3);
  IP4_ADDR(&netmask, NETMASK_ADDR0, NETMASK_ADDR1 , NETMASK_ADDR2, NETMASK_ADDR3);
  IP4_ADDR(&gw, GW_ADDR0, GW_ADDR1, GW_ADDR2, GW_ADDR3);
#endif

  /* Create tcp_ip stack thread */
  tcpip_init( NULL, NULL );

  /* Adds network interface to the netif_list */
  netif_add(&xnetif, &ipaddr, &netmask, &gw, NULL, &ethernetif_init, &tcpip_input);

  /*  Registers the default network interface.*/
  netif_set_default(&xnetif);

  /* If callback enabled */
#if LWIP_NETIF_STATUS_CALLBACK == 1
  /* Initialize interface status change callback */
  netif_set_status_callback(&xnetif, ETH_NETIF_STATUS_CB_FUNCTION);
#endif

}
#endif

